# Hello, Marvin!
![](../../img/heroes/it_hates_me_hero.png)


Marvin is a lightweight framework for building AI-powered software that's reliable, scalable, and easy to trust. It's designed primarily for AI engineers: users who want to deploy cutting-edge AI to build powerful new features and applications. 

Large Language Models (LLMs) are pretty cool, but let's face it, they can be a headache to integrate. So, we decided to take the fuss out of the process. Marvin is our answer to the challenge: a neat, flexible tool that works as hard as you do.

Sometimes the most challenging part of working with generative AI is remembering that it's not magic; it's software. It's new, it's nondeterministic, and it's incredibly powerful, but it's still software: parameterized API calls that can trigger dependent actions (and just might talk like a pirate). Marvin's goal is to bring the best practices of building dependable, observable software to the frontier of generative AI. As the team behind [Prefect](https://github.com/prefecthq/prefect), which does something very similar for data engineers, we've poured years of open-source developer tool experience (and a few hard-won lessons!) into Marvin's design.

## Developer Experience

Above all else, Marvin is focused on a rock-solid developer experience. It's ergonomic and opinionated at every layer, but also incrementally adoptable so you can use it as much or as little as you like. It’s a Swiss Army Knife, not a kitchen sink. It’s familiar. It feels like the library you’d write if you had the time: simple, accessible, portable LLM abstractions that you can quickly deploy in your application, whether you’re doing straightforward NLP or building a full-featured autonomous agent.

Marvin prioritizes a developer experience focused on speed and reliability. It's built with type-safety and observability as first-class citizens. Its abstractions are Pythonic, simple, and self-documenting. These core primitives let us build surprisingly complex agentic software without sacrificing control:

🧩 **AI Models** for structuring text into type-safe schemas

🏷️ **AI Classifiers** for bulletproof classification and routing

🪄 **AI Functions** for complex business logic and transformations

🤝 **AI Applications** for interactive use and persistent state

## Ambient AI

With Marvin, we’re taking the first steps on a journey to deliver [Ambient AI](https://twitter.com/DrJimFan/status/1657782710344249344): omnipresent but unobtrusive autonomous routines that act as persistent translators for noisy, real-world data. Ambient AI makes unstructured data universally accessible to traditional software, allowing the entire software stack to embrace AI technology without interrupting the development workflow. Marvin brings simplicity and stability to AI engineering through abstractions that are reliable and easy to trust. 

## What Makes Marvin Different?

There's no shortage of tools and libraries out there for integrating LLMs into your software. So what makes Marvin different? In addition to a relentless focus on incrementally-adoptable, familiar abstractions, Marvin embraces five pillars:

1. **User-Centric Design:** We built Marvin with you in mind. It's not just about what it does, but how it does it. Marvin is designed to be as user-friendly as possible, with a focus on an easy, intuitive experience. Whether you're a coding expert or just starting, Marvin works for you.

1. **Flexibility:** Marvin is built to adapt to your needs, not the other way around. You can use as much or as little of Marvin as you need. Need a full suite of LLM integration tools? We've got you covered. Just need a component or two for a quick project? Marvin can do that too.

1. **Community Driven:** Marvin isn't just a tool, it's a community. We value feedback and collaboration from users like you. We're always learning, iterating, and improving based on what our community tells us.

1. **Velocity:** We believe that getting started should be quick and easy. That's why with Marvin, you can get up and running in no time. Marvin is not here to do everything for you, it's here to eliminate the the most cumbersome parts of working with AI in order to accelerate your ability to take advantage of it.

1. **Open-Source:** Marvin is fully open-source, which means it's not only free to use, but you're also free to modify and adapt as you see fit. The [Prefect](https://www.prefect.io) team has years of open-source experience and is fully committed to supporting Marvin as an open-source product. We believe in the power of collective intelligence, and we're excited to see what you can create. 

Marvin's 1.0 release reflects our confidence that its core abstractions are locked-in. And why wouldn't they be? They're the same interfaces you use every day: Python functions, classes, enums, and Pydantic models. Our next objectives are leveraging these primitives to build production deployment patterns and an observability platform.


If our mission is exciting to you and you’d like to build Marvin with us, [join our community](/src/community)!